CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C      Code for calculating the evolution of the Sivers function 
C      from the Torino fits  (Eur.Phys. J. A39,89(2009))
C      with evolution implemented following arXiv:1110.6428.
C
C
C      TorinoSivers(xval,ktval,qval,fl) is a callable 
C          function WITH THE CORRECT NORMALIZATION that
C              returns the Sivers function with:
C                   xval = Bjorken-x
C                   ktval = transvserse momentum (in GeV)
C                   qval = hard scale (in GeV)
C                   fl = flavor (1=up, 2=down)
C
C
C	 The Sivers functions here are Gaussians obtained
C      with parameters depending on q and parametrized 
C      previously with the FindFit routine in 
C      Wolfram Mathematica.         
C	   
C
C      The normalization of the Torino fits uses the 
C	 collinear pdfs.  In this case we link to 
C	 the MSTW pdfs.
C       
C
C      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      real*8 function TorinoSivers(xval,ktval,qval,fl)
      implicit none
            INTEGER iset,ieigen,neigen,ix,nx,iq,nq,flav,
     &     alphaSorder,alphaSnfmax,lentrim
      PARAMETER(neigen=20)      ! number of eigenvectors
      DOUBLE PRECISION x,q,upv,dnv,usea,dsea,str,sbar,
     &     chm,cbar,bot,bbar,glu,phot,upv1,dnv1,usea1,dsea1,str1,sbar1,
     &     chm1,cbar1,bot1,bbar1,glu1,phot1,xphoton,
     &     GetOnePDF,xpdf(-6:6),xf,xfp,xfm,xmin,xmax,q2min,q2max,
     &     summin,summax,sum,distance,tolerance,
     &     mCharm,mBottom,alphaSQ0,alphaSMZ,ALPHAS
      CHARACTER prefix*50,prefix1*55,flavours(-5:5)*10,
     &     xfilename*50,qfilename*50
      COMMON/mstwCommon/distance,tolerance,
     &     mCharm,mBottom,alphaSQ0,alphaSMZ,alphaSorder,alphaSnfmax
      DATA flavours /"bbar","cbar","sbar","ubar","dbar","glu",
     &     "dn","up","str","chm","bot"/

      real*8 CF,TF,C1,MP,pi,e 
      real*8 xval,g2val,Qinitval,Q0val,qval,ktval
      real*8 x0
      real*8 kt2zeroval
      real*8 sivvalup,sivvaldown
      real*8 TorinoUpSivers, TorinoDownSivers
      real*8 M1,Nup,Nup0,Ndown,Ndown0,fzerup
      real*8 alphau,alphad,beta,Nu,Nd,kt2
      real*8 fzerup0,fzerdown,fzerdown0
      real*8 fonepertup,fonepertup0
      real*8 fonepertdown,fonepertdown0
      real*8 normup,normdown
      integer num, fl

C	INPUT AND INITIALIZE MSTW PDFS AND ALPHAS.

      prefix = "mstw2008lo" ! prefix for the grid files
      iset = 0
      x = 1.d-3
      q = 1.d1
C--      WRITE(6,*) "x = ",x,", q = ",q
      flav = 2

      CALL GetAllPDFsAlt(prefix,iset,x,q,xpdf,xphoton)


      g2val = 0.68d0
      Qinitval = Sqrt(2.4d0)
      Q0val = 3.2d0
      x0 = 0.1d0
      CF=4.0d0/3.0d0
      TF=1.0d0/2.0d0
      C1=1.123d0
      pi=3.1415926536d0
      MP=0.938d0
      e=2.7182818284d0



C	BASIC PARAMETERS FOR THE TORINO NORMALIZATION.
C	SEE REF.... 


      Nu = 0.35d0
      Nd = 0.90d0
      alphau = 0.73d0
      alphad = 1.08d0
      beta = 3.46d0
      M1 = Sqrt(0.34d0)
      kt2 = 0.25d0
      MP = 0.938d0


      Nup = Nu*(xval**alphau)*((1.0d0 - xval)**(beta))
     &*((alphau + beta)**(alphau + beta))
     &/((alphau**alphau)*(beta**beta))

      Nup0 = Nu*(x0**alphau)*((1.0d0 - x0)**(beta))
     &*((alphau + beta)**(alphau + beta))
     &/((alphau**alphau)*(beta**beta))
   
      Ndown = Nd*(xval**alphad)*((1.0d0 - xval)**(beta))
     &*((alphad + beta)**(alphad + beta))
     &/((alphad**alphad)*(beta**beta))

      Ndown0 = Nd*(x0**alphad)*((1.0d0 - x0)**(beta))
     &*((alphad + beta)**(alphad + beta))
     &/((alphad**alphad)*(beta**beta))

      kt2zeroval = (M1**(2.0d0))*(kt2**(2.0d0))
     &/(M1**(2.0d0) + kt2)
     &-2.0d0*g2val*log(Qinitval/Q0val)

      fzerup = GetOnePDF(prefix,iset,xval,2.5111d0,2)/xval
      fzerup0 = GetOnePDF(prefix,iset,x0,2.5111d0,2)/x0

      fzerdown = GetOnePDF(prefix,iset,xval,2.5111d0,1)/xval
      fzerdown0 = GetOnePDF(prefix,iset,x0,2.5111d0,1)/x0
    
      fonepertup = -1.0d0*(4.0d0*pi*MP*Sqrt(2.0d0*e**1.0d0)
     &/(M1*pi*kt2))*Nup*fzerup*(((kt2zeroval/2.0d0)
     &+ g2val*log(Qinitval/Q0val))**(2.0d0))/kt2zeroval

      fonepertup0 = -1.0d0*(4.0d0*pi*MP*Sqrt(2.0d0*e**1.0d0)
     &/(M1*pi*kt2))*Nup0*fzerup0*(((kt2zeroval/2.0d0)
     &+ g2val*log(Qinitval/Q0val))**(2.0d0))/kt2zeroval

      fonepertdown = -1.0d0*(4.0d0*pi*MP*Sqrt(2.0d0*e**1.0d0)
     &/(M1*pi*kt2))*Ndown*fzerdown*(((kt2zeroval/2.0d0)
     &+ g2val*log(Qinitval/Q0val))**(2.0d0))/kt2zeroval

      fonepertdown0 = -1.0d0*(4.0d0*pi*MP*Sqrt(2.0d0*e**1.0d0)
     &/(M1*pi*kt2))*Ndown0*fzerdown0*(((kt2zeroval/2.0d0)
     &+ g2val*log(Qinitval/Q0val))**(2.0d0))/kt2zeroval

      normup = (-MP/(2.0d0*ktval*x0))*(fonepertup/fonepertup0)
      normdown = (MP/(2.0d0*ktval*x0))*(fonepertdown/fonepertdown0)      	

      sivvalup = normup*TorinoUpSivers(xval,ktval,qval)
      
      sivvaldown = normdown
     &*TorinoDownSivers(xval,ktval,qval)
      
      if(fl.eq.1) then
      TorinoSivers = sivvalup
      else
      if(fl.eq.2) then
      TorinoSivers = sivvaldown 
      else 
      PRINT *, 'Unrecognized Flavor'
      endif
      endif

      end
     

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
  
      real*8 function TorinoUpSivers(x,kt,q)
      implicit none
      real*8 x,kt,q
      real*8 CF,TF,C1,bmax,MP,pi,e
      real*8 xval,g2val,Qinitval,Q0val,x0
      real*8 kt2zeroval,gaussnormfitlow
      real*8 gaussnormfithigh,gaussparfit
      real*8 fonepert,fonepert0
      CF=4.0d0/3.0d0
      TF=1.0d0/2.0d0
      C1=1.123d0
      pi=3.1415926536d0
      MP=0.938d0
      e=2.7182818284d0  
      gaussnormfitlow = 0.856642d0/(-7.92053d0*q + 5.3952*(q**2.0d0) 
     &- 0.0307745d0*q**(4.0d0) - 0.00134312d0*q**(6.0d0))
      gaussnormfithigh = 0.00020356d0 + 1635.84d0/(-47186.2d0*Sqrt(q) 
     &+ 43453.6d0*q)
      gaussparfit = 1.4556d0/Log(0.513906d0*(q**2.0d0))
      if(q.lt.5.0d0) then
      TorinoUpSivers = gaussnormfitlow*kt*e**(-gaussparfit*kt**(2.0d0)) 
      else
      TorinoUpSivers = gaussnormfithigh*kt*e**(-gaussparfit*kt**(2.0d0))
      end if      

      return
      end
      


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      real*8 function TorinoDownSivers(x,kt,q)
      implicit none
      real*8 x,kt,q
      real*8 CF,TF,C1,bmax,MP,pi,e
      real*8 xval,g2val,Qinitval,Q0val,x0
      real*8 kt2zeroval,gaussnormfitlow
      real*8 gaussnormfithigh,gaussparfit
      real*8 fonepert,fonepert0
      CF=4.0d0/3.0d0
      TF=1.0d0/2.0d0
      C1=1.123d0
      pi=3.1415926536d0
      MP=0.938d0
      e=2.7182818284d0
      g2val = 0.68d0
      Qinitval = Sqrt(2.4d0)
      Q0val = 3.2d0
      x0 = 0.1d0  
      gaussnormfitlow = 4.67378d0/(-32.0522d0*q 
     &+ 21.8329d0*(q**2.0d0) - 0.124536d0*(q**4.0d0) 
     &- 0.00543525d0*(q**6.0))
      gaussnormfithigh = 0.000274447d0 
     &+ 746.554d0/(-15972.4d0*Sqrt(q) + 14708.9d0*q)	
      gaussparfit = 1.4556d0/Log(0.513906d0*(q**2.0d0))
      if(q.lt.5.0d0) then
      TorinoDownSivers = gaussnormfitlow
     &*kt*e**(-gaussparfit*kt**(2.0d0)) 
      else
      TorinoDownSivers = gaussnormfithigh
     &*kt*e**(-gaussparfit*kt**(2.0d0))
      end if      

      return
      end 

